import { get, isNumber } from 'lodash';
var localeTemp = get(window, 'g_config.locale') || 'zh_CN';
var locale = /^zh[-_]cn$/i.test(localeTemp) ? 'zh_CN' : 'en_US';
var varList = get(window, "arrangement[" + locale + "]") || {};
export var i18n = function i18n(key) {
  return varList[key] || key;
};
export var formatCssNumber = function formatCssNumber(value) {
  var val = +("" + value).replace('px', '');
  return isNaN(val) ? 0 : val;
};
/**
 * 判断是否合法的透明度值，范围是 [0, 1]
 * @param value Number 透明度值
 * @returns Boolean
 */

export var isAlphaNumber = function isAlphaNumber(value) {
  return isNumber(value) && value >= 0 && value <= 1;
};
/**
 * 获取元素的阴影样式
 * @param style IShadowStyle 阴影样式
 * @returns IShadowStyle
 */

export var getShadowStyle = function getShadowStyle(style) {
  var shadowOffsetX = style.shadowOffsetX,
      shadowOffsetY = style.shadowOffsetY,
      shadowBlur = style.shadowBlur,
      shadowColor = style.shadowColor;
  var shadowStyle = {};

  if (shadowColor) {
    Object.assign(shadowStyle, {
      shadowColor: shadowColor,
      shadowBlur: formatCssNumber(shadowBlur || 0),
      shadowOffsetX: formatCssNumber(shadowOffsetX || 0),
      shadowOffsetY: formatCssNumber(shadowOffsetY || 0)
    });
  }

  return shadowStyle;
};
/**
 * 获取元素的透明度样式
 * @param style IOpacityStyle 透明度样式
 * @returns IOpacityStyle
 */

export var getOpacityStyle = function getOpacityStyle(style) {
  var opacityStyle = {};
  Object.keys(style).forEach(function (key) {
    var value = style[key];

    if (isAlphaNumber(value)) {
      opacityStyle[key] = value;
    }
  });
  return opacityStyle;
};