import { isString, memoize, values } from '@antv/util';
var ctx;
/**
 * 获取 canvas context
 */

export function getCanvasContext() {
  if (!ctx) {
    ctx = document.createElement('canvas').getContext('2d');
  }

  return ctx;
}
/**
 * 计算文本在画布中的宽高
 * @param text 文本
 * @param font 字体
 */

export var measureText = memoize(function (text, font) {
  if (font === void 0) {
    font = {};
  }

  var _font = font,
      _font$fontSize = _font.fontSize,
      fontSize = _font$fontSize === void 0 ? 12 : _font$fontSize,
      _font$fontFamily = _font.fontFamily,
      fontFamily = _font$fontFamily === void 0 ? 'PingFang SC' : _font$fontFamily,
      fontWeight = _font.fontWeight,
      fontStyle = _font.fontStyle,
      fontVariant = _font.fontVariant; // return {
  //   // 使用近似的文本高度
  //   // https://stackoverflow.com/questions/1134586/how-can-you-find-the-height-of-text-on-an-html-canvas
  //   height: fontSize * 1.2,
  //   width: fontSize * text.length,
  // };

  var ctx = getCanvasContext();

  if (ctx) {
    // @see https://developer.mozilla.org/zh-CN/docs/Web/CSS/font
    ctx.font = [fontStyle, fontWeight, fontVariant, fontSize + "px", fontFamily].join(' ');
    var metrics = ctx.measureText(isString(text) ? text : ''); // metrics
    // actualBoundingBoxAscent: 8.63671875
    // actualBoundingBoxDescent: 2.654296875
    // actualBoundingBoxLeft: -1.025390625
    // actualBoundingBoxRight: 98.595703125
    // fontBoundingBoxAscent: 11
    // fontBoundingBoxDescent: 3
    // width: 99.380859375

    return {
      // 使用近似的文本高度
      // https://stackoverflow.com/questions/1134586/how-can-you-find-the-height-of-text-on-an-html-canvas
      height: fontSize * 1.2,
      width: metrics.width
    };
  }

  return {
    height: 0,
    width: 0
  };
}, function (text, font) {
  if (font === void 0) {
    font = {};
  }

  return [text].concat(values(font)).join('');
});