"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

exports.__esModule = true;
exports["default"] = void 0;

var _extends2 = _interopRequireDefault(require("@babel/runtime/helpers/extends"));

var React = _interopRequireWildcard(require("react"));

var _renderer = require("../../renderer");

var _util = require("@antv/util");

function _getRequireWildcardCache(nodeInterop) { if (typeof WeakMap !== "function") return null; var cacheBabelInterop = new WeakMap(); var cacheNodeInterop = new WeakMap(); return (_getRequireWildcardCache = function _getRequireWildcardCache(nodeInterop) { return nodeInterop ? cacheNodeInterop : cacheBabelInterop; })(nodeInterop); }

function _interopRequireWildcard(obj, nodeInterop) { if (!nodeInterop && obj && obj.__esModule) { return obj; } if (obj === null || typeof obj !== "object" && typeof obj !== "function") { return { "default": obj }; } var cache = _getRequireWildcardCache(nodeInterop); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var key in obj) { if (key !== "default" && Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } newObj["default"] = obj; if (cache) { cache.set(obj, newObj); } return newObj; }

function isWordSplitterChar(_char, index, str, wordSplitter) {
  var isRegExp = Object.prototype.toString.call(wordSplitter) === '[object RegExp]'; // 自定义单词分割器，正则模式

  if (isRegExp) {
    return wordSplitter.test(_char);
  } // 自定义单词分割器，函数模式


  if (typeof wordSplitter === 'function') {
    return wordSplitter(_char, index, str);
  } // 默认的单词分割器是非字母、数字、下划线


  return /[^\w]/.test(_char);
}

function split2Word(str, wordSplitter) {
  var len = str.length;
  var words = [];
  var word = '';

  for (var i = 0; i < len; i++) {
    var _char2 = str.charAt(i);

    if (isWordSplitterChar(_char2, i, str, wordSplitter)) {
      if (word) {
        words.push(word);
        word = '';
      }

      words.push(_char2);
    } else {
      word += _char2;
    }
  }

  if (word) {
    words.push(word);
  }

  return words;
}

function calcWordWidth(word, fontSize) {
  var pattern = new RegExp("[\u4E00-\u9FA5]+"); // distinguish the Chinese charactors and letters

  var width = 0;
  var letters = word.split('');

  for (var i = 0, len = letters.length; i < len; i++) {
    var letter = letters[i];

    if (pattern.test(letter)) {
      // Chinese charactors
      width += fontSize;
    } else {
      // get the width of single letter according to the fontSize
      width += (0, _util.measureTextWidth)(letter, fontSize);
    }
  }

  return width;
}
/**
 * format the string
 * @param {string} str The origin string
 * @param {number} maxWidth max width
 * @param {number} fontSize font size
 * @param {boolean} keepWord word break
 * @return {string} the processed result
 */


var fittingString = function fittingString(str, maxWidth, fontSize, keepWord, wordSplitter) {
  var currentWidth = 0;
  var sentence = '';
  var sentences = [];
  var words = keepWord ? split2Word(str, wordSplitter) : str.split('');

  for (var i = 0, len = words.length; i < len; i++) {
    var word = words[i];
    var wordWidth = calcWordWidth(word, fontSize);

    if (currentWidth + wordWidth > maxWidth) {
      sentences.push(sentence);
      currentWidth = 0;
      sentence = '';
    }

    currentWidth += wordWidth;
    sentence += word;
  }

  if (sentence) {
    sentences.push(sentence);
  }

  return sentences;
};

var Textarea = function Textarea(props) {
  var _props$text = props.text,
      text = _props$text === void 0 ? '' : _props$text,
      _props$placeholder = props.placeholder,
      placeholder = _props$placeholder === void 0 ? '' : _props$placeholder,
      _props$wordBreak = props.wordBreak,
      wordBreak = _props$wordBreak === void 0 ? false : _props$wordBreak,
      wordSplitter = props.wordSplitter,
      _props$style = props.style,
      style = _props$style === void 0 ? {} : _props$style,
      _props$wrapperStyle = props.wrapperStyle,
      wrapperStyle = _props$wrapperStyle === void 0 ? {} : _props$wrapperStyle,
      maxLineCount = props.maxLineCount,
      name = props.name;
  var fontSize = style.fontSize || 14;
  var lineHeight = style.lineHeight || fontSize * 1.2;
  var height = style.height || lineHeight;
  var maxWidth = style.maxWidth || wrapperStyle.maxWidth || 200;
  var textLines = fittingString(text || placeholder, maxWidth, fontSize, !wordBreak, wordSplitter);

  if (!maxLineCount) {
    return /*#__PURE__*/React.createElement(_renderer.Rect, {
      name: name,
      style: (0, _extends2["default"])({}, wrapperStyle)
    }, textLines.map(function (txt, index) {
      return /*#__PURE__*/React.createElement(_renderer.Text, {
        name: "" + name + index,
        style: (0, _extends2["default"])({}, style, {
          lineHeight: lineHeight,
          height: height
        })
      }, txt);
    }));
  } else {
    // 正常渲染前maxLine行
    var normalTextLines = textLines.splice(0, maxLineCount - 1); // maxLine后拼接成一行省略渲染

    var ellipsisTextLine = textLines.join();
    return /*#__PURE__*/React.createElement(_renderer.Rect, {
      name: name,
      style: (0, _extends2["default"])({}, wrapperStyle)
    }, normalTextLines.map(function (txt, index) {
      return /*#__PURE__*/React.createElement(_renderer.Text, {
        name: "" + name + index,
        style: (0, _extends2["default"])({}, style, {
          lineHeight: lineHeight,
          height: height
        })
      }, txt);
    }), ellipsisTextLine !== '' && /*#__PURE__*/React.createElement(_renderer.Text, {
      name: "" + name + (maxLineCount - 1),
      style: (0, _extends2["default"])({}, style, {
        maxWidth: maxWidth - 2 * fontSize,
        whiteSpace: 'nowrap',
        textOverflow: 'ellipsis',
        overflow: 'hidden'
      })
    }, ellipsisTextLine));
  }
};

var _default = Textarea;
exports["default"] = _default;