"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

exports.__esModule = true;
exports.updateImage = exports["default"] = void 0;

var _extends2 = _interopRequireDefault(require("@babel/runtime/helpers/extends"));

var _util = require("../../util");

var renderImage = function renderImage(group) {
  return function (node) {
    var _node$box = node.box,
        left = _node$box.left,
        top = _node$box.top,
        width = _node$box.width,
        height = _node$box.height;
    var _node$props = node.props,
        clip = _node$props.clip,
        src = _node$props.src,
        draggable = _node$props.draggable;
    var _node$style = node.style,
        shadowOffsetX = _node$style.shadowOffsetX,
        shadowOffsetY = _node$style.shadowOffsetY,
        shadowBlur = _node$style.shadowBlur,
        shadowColor = _node$style.shadowColor,
        opacity = _node$style.opacity,
        fillOpacity = _node$style.fillOpacity,
        strokeOpacity = _node$style.strokeOpacity,
        cursor = _node$style.cursor,
        zIndex = _node$style.zIndex;
    var nodeDraggable = node.draggable || draggable;
    nodeDraggable = nodeDraggable !== false; // shadow

    var shadowStyle = (0, _util.getShadowStyle)({
      shadowOffsetX: shadowOffsetX,
      shadowOffsetY: shadowOffsetY,
      shadowBlur: shadowBlur,
      shadowColor: shadowColor
    }); // opacity

    var opacityStyle = (0, _util.getOpacityStyle)({
      opacity: opacity,
      fillOpacity: fillOpacity,
      strokeOpacity: strokeOpacity
    });
    var image = group.addShape('image', {
      attrs: (0, _extends2["default"])({
        // Common Attr
        name: "g-react-item-" + (node.name || Math.random())
      }, shadowStyle, opacityStyle, {
        cursor: cursor,
        // Image Attr
        x: left,
        y: top,
        width: width,
        height: height,
        img: src
      }),
      draggable: nodeDraggable,
      zIndex: zIndex || 0
    });

    if (clip === 'circle') {
      // clip circle, 3.2.10暂不支持
      image.setClip({
        type: 'circle',
        // 支持 circle、rect、ellipse、Polygon 及自定义 path clip
        attrs: {
          r: width / 2,
          x: left + width / 2,
          y: top + height / 2
        }
      });
    }

    return node;
  };
};

var updateImage = function updateImage(targetElement) {
  return function (newProps, newStyle, oldVNode) {
    // 优先使用新的clip属性
    var clip = newProps.clip || oldVNode.props.clip;
    var newAttr = (0, _extends2["default"])({}, newProps);

    if (newProps.left !== undefined) {
      newAttr.x = newProps.left;
    }

    if (newProps.top !== undefined) {
      newAttr.y = newProps.top;
    }

    if (Object.keys(newStyle).length > 0) {
      var shadowOffsetX = newStyle.shadowOffsetX,
          shadowOffsetY = newStyle.shadowOffsetY,
          shadowBlur = newStyle.shadowBlur,
          shadowColor = newStyle.shadowColor,
          opacity = newStyle.opacity,
          fillOpacity = newStyle.fillOpacity,
          strokeOpacity = newStyle.strokeOpacity; // shadow

      var shadowStyle = (0, _util.getShadowStyle)({
        shadowOffsetX: shadowOffsetX,
        shadowOffsetY: shadowOffsetY,
        shadowBlur: shadowBlur,
        shadowColor: shadowColor
      }); // opacity

      var opacityStyle = (0, _util.getOpacityStyle)({
        opacity: opacity,
        fillOpacity: fillOpacity,
        strokeOpacity: strokeOpacity
      }); // assign

      newAttr = (0, _extends2["default"])({}, newAttr, shadowStyle, opacityStyle);
    }

    targetElement.attr(newAttr);

    if (clip === 'circle') {
      // clip circle, 3.2.10暂不支持
      var width = newAttr.width || oldVNode.box.width;
      var height = newAttr.height || oldVNode.box.height;
      var left = newAttr.left || oldVNode.box.left;
      var top = newAttr.top || oldVNode.box.top;
      targetElement.setClip({
        type: 'circle',
        // 支持 circle、rect、ellipse、Polygon 及自定义 path clip
        attrs: {
          r: width / 2,
          x: left + width / 2,
          y: top + height / 2
        }
      });
    }
  };
};

exports.updateImage = updateImage;
var _default = renderImage;
exports["default"] = _default;