"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

exports.__esModule = true;
exports.updateRect = exports.renderRect = exports["default"] = void 0;

var _extends2 = _interopRequireDefault(require("@babel/runtime/helpers/extends"));

var _util = require("../../util");

var _lodash = require("lodash");

var renderRect = function renderRect(nodeGroup) {
  return function (node) {
    var _node$box = node.box,
        top = _node$box.top,
        left = _node$box.left,
        width = _node$box.width,
        height = _node$box.height;
    var _node$props = node.props,
        link = _node$props.link,
        draggable = _node$props.draggable;
    var nodeDraggable = node.draggable || draggable;
    nodeDraggable = nodeDraggable !== false;
    var _node$style = node.style,
        backgroundColor = _node$style.backgroundColor,
        borderTopLeftRadius = _node$style.borderTopLeftRadius,
        borderBottomRightRadius = _node$style.borderBottomRightRadius,
        borderBottomLeftRadius = _node$style.borderBottomLeftRadius,
        borderTopRightRadius = _node$style.borderTopRightRadius,
        borderColor = _node$style.borderColor,
        borderLeftWidth = _node$style.borderLeftWidth,
        borderLeftColor = _node$style.borderLeftColor,
        shadowOffsetX = _node$style.shadowOffsetX,
        shadowOffsetY = _node$style.shadowOffsetY,
        shadowBlur = _node$style.shadowBlur,
        shadowColor = _node$style.shadowColor,
        opacity = _node$style.opacity,
        fillOpacity = _node$style.fillOpacity,
        strokeOpacity = _node$style.strokeOpacity,
        cursor = _node$style.cursor,
        rotate = _node$style.rotate,
        translate = _node$style.translate,
        zIndex = _node$style.zIndex; // FIXME: HACK

    var lineWidth = borderLeftWidth || 0;
    var stroke = borderLeftColor || '';
    var radius = [borderTopLeftRadius || 0, borderTopRightRadius || 0, borderBottomRightRadius || 0, borderBottomLeftRadius || 0]; // shadow

    var shadowStyle = (0, _util.getShadowStyle)({
      shadowOffsetX: shadowOffsetX,
      shadowOffsetY: shadowOffsetY,
      shadowBlur: shadowBlur,
      shadowColor: shadowColor
    }); // opacity

    var opacityStyle = (0, _util.getOpacityStyle)({
      opacity: opacity,
      fillOpacity: fillOpacity,
      strokeOpacity: strokeOpacity
    });
    var rect = nodeGroup.addShape('rect', {
      attrs: (0, _extends2["default"])({
        // Common Attr
        name: "g-react-item-" + (node.name || Math.random()),
        fill: backgroundColor,
        stroke: stroke,
        lineWidth: lineWidth
      }, shadowStyle, opacityStyle, {
        cursor: cursor || link && 'pointer',
        // Rect Attr
        x: left,
        y: top,
        width: width,
        height: height,
        radius: radius,
        // Extra Attr
        link: link
      }),
      draggable: nodeDraggable,
      zIndex: zIndex || 0
    });

    if (rotate) {
      rect.rotate(rotate);
    }

    if (translate && (0, _lodash.isNumber)(translate.x) && (0, _lodash.isNumber)(translate.y)) {
      rect.translate(translate.x, translate.y);
    }

    return node;
  };
};

exports.renderRect = renderRect;

var updateRect = function updateRect(targetElement) {
  return function (newProps, newStyle) {
    var newAttr = (0, _extends2["default"])({}, newProps);

    if (newProps.left !== undefined) {
      newAttr.x = newProps.left;
    }

    if (newProps.top !== undefined) {
      newAttr.y = newProps.top;
    }

    if (Object.keys(newStyle).length > 0) {
      var backgroundColor = newStyle.backgroundColor,
          borderTopLeftRadius = newStyle.borderTopLeftRadius,
          borderBottomRightRadius = newStyle.borderBottomRightRadius,
          borderBottomLeftRadius = newStyle.borderBottomLeftRadius,
          borderTopRightRadius = newStyle.borderTopRightRadius,
          borderLeftWidth = newStyle.borderLeftWidth,
          borderLeftColor = newStyle.borderLeftColor,
          shadowOffsetX = newStyle.shadowOffsetX,
          shadowOffsetY = newStyle.shadowOffsetY,
          shadowBlur = newStyle.shadowBlur,
          shadowColor = newStyle.shadowColor,
          opacity = newStyle.opacity,
          fillOpacity = newStyle.fillOpacity,
          strokeOpacity = newStyle.strokeOpacity; // FIXME: HACK

      var lineWidth = borderLeftWidth || 0;
      var stroke = borderLeftColor || '';
      var radius = [borderTopLeftRadius || 0, borderTopRightRadius || 0, borderBottomRightRadius || 0, borderBottomLeftRadius || 0]; // shadow

      var shadowStyle = (0, _util.getShadowStyle)({
        shadowOffsetX: shadowOffsetX,
        shadowOffsetY: shadowOffsetY,
        shadowBlur: shadowBlur,
        shadowColor: shadowColor
      }); // opacity

      var opacityStyle = (0, _util.getOpacityStyle)({
        opacity: opacity,
        fillOpacity: fillOpacity,
        strokeOpacity: strokeOpacity
      });
      newAttr = (0, _extends2["default"])({}, newAttr, {
        fill: backgroundColor,
        radius: radius,
        stroke: stroke,
        lineWidth: lineWidth
      }, shadowStyle, opacityStyle);
    }

    targetElement.attr(newAttr);
  };
};

exports.updateRect = updateRect;
var _default = renderRect;
exports["default"] = _default;