"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

exports.__esModule = true;
exports.updateText = exports["default"] = void 0;

var _extends2 = _interopRequireDefault(require("@babel/runtime/helpers/extends"));

var R = _interopRequireWildcard(require("ramda"));

var _measureText = require("../../util/measureText");

var _util = require("@antv/util");

function _getRequireWildcardCache(nodeInterop) { if (typeof WeakMap !== "function") return null; var cacheBabelInterop = new WeakMap(); var cacheNodeInterop = new WeakMap(); return (_getRequireWildcardCache = function _getRequireWildcardCache(nodeInterop) { return nodeInterop ? cacheNodeInterop : cacheBabelInterop; })(nodeInterop); }

function _interopRequireWildcard(obj, nodeInterop) { if (!nodeInterop && obj && obj.__esModule) { return obj; } if (obj === null || typeof obj !== "object" && typeof obj !== "function") { return { "default": obj }; } var cache = _getRequireWildcardCache(nodeInterop); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var key in obj) { if (key !== "default" && Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } newObj["default"] = obj; if (cache) { cache.set(obj, newObj); } return newObj; }

var fittingString = function fittingString(str, maxWidth, fontSize) {
  var currentWidth = 0;
  var res = str;
  var insertIndex = [];
  var pattern = new RegExp("[\u4E00-\u9FA5]+"); // distinguish the Chinese charactors and letters

  var tempArr = str.split('');
  tempArr.forEach(function (letter, i) {
    if (currentWidth > maxWidth) return;

    if (pattern.test(letter)) {
      // Chinese charactors
      currentWidth += fontSize;
    } else {
      // get the width of single letter according to the fontSize
      currentWidth += (0, _util.measureTextWidth)(letter, fontSize);
    }

    if (currentWidth > maxWidth) {
      insertIndex.push(i);
      currentWidth = 0;
    }
  });
  insertIndex.forEach(function (item) {
    tempArr.splice(item, 0, '\n');
  });
  res = tempArr.join('');
  return res;
};

function getSubText(text, style, remainWidth) {
  var splitIdx = Math.floor(text.length / 2);
  var prevStr = text.substring(0, splitIdx);
  var nextStr = text.substring(splitIdx);
  var prevInfo = (0, _measureText.measureText)(prevStr, style);
  var prevWidth = prevInfo.width || 0;
  var matchedStr = ''; // 如第一段仍较长，则继续进行拆分

  if (text && text.length > 2) {
    if (prevWidth > remainWidth) {
      matchedStr = getSubText(prevStr, style, remainWidth);
    } else {
      // 如第一段不够长，则处理第二段，叠加宽度
      matchedStr = prevStr + getSubText(nextStr, style, remainWidth - prevWidth);
    }
  }

  return matchedStr;
}

function clipText(text, style, maxWidth, config) {
  if (config === void 0) {
    config = {};
  }

  var _config = config,
      _config$indicator = _config.indicator,
      indicator = _config$indicator === void 0 ? "\u2026" : _config$indicator;
  var textInfo = (0, _measureText.measureText)(text, style); // indicator: \u2026 -> ...
  // 进行 clip

  if (textInfo.width > maxWidth) {
    if (style.textOverflow === 'ellipsis') {
      // 截断...
      var indicatorInfo = (0, _measureText.measureText)(indicator, style); // 基于二分法计算字符宽度
      // clip text

      var subText = getSubText(text, style, maxWidth - indicatorInfo.width);
      return subText + indicator;
    } else {
      // 换行
      return fittingString(text, maxWidth, style.fontSize || 14);
    }
  }

  return text;
}

var renderText = function renderText(nodeGroup) {
  return function (node) {
    var _node$box = node.box,
        top = _node$box.top,
        left = _node$box.left;
    var _node$props = node.props,
        link = _node$props.link,
        cursor = _node$props.cursor,
        name = _node$props.name,
        draggable = _node$props.draggable; // 目前简化处理,仅处理第一个 text instance

    if (node && node.children && node.children[0] && node.children[0].value) {
      var text = node.children[0].value;
      var attributes = node.style || {};
      var textColor = attributes.color || '#000';
      var nodeDraggable = node.draggable || draggable;
      nodeDraggable = nodeDraggable !== false;
      var textStyle = {};

      if (attributes.fontSize) {
        Object.assign(textStyle, {
          fontSize: attributes.fontSize
        });
      }

      var commonStyle = {};

      if (attributes.cursor) {
        Object.assign(commonStyle, {
          cursor: attributes.cursor
        });
      }

      var showTip = false;
      var resultText = text;

      if (attributes.maxWidth || attributes.width) {
        resultText = clipText(text, attributes, attributes.maxWidth); // console.log(attributes, text, resultText, measureText(resultText, attributes));

        node.box.height = 48;
      }

      if (resultText !== text) {
        showTip = true;
      }

      nodeGroup.addShape('text', {
        attrs: (0, _extends2["default"])({}, R.omit(['fontSize'], attributes), {
          // Common Attr
          name: "g-react-item-" + (node.name || Math.random()),
          fill: textColor,
          cursor: cursor || link && 'pointer'
        }, commonStyle, {
          // 暂未支持，需判断是否可以和必要实现
          // stroke,lineWidth,lineDash,shadowColor,shadowBlur,shadowOffsetX,shadowOffsetY,opacity,fillOpacity,strokeOpacity
          // Text Attr
          text: resultText,
          x: left,
          y: top,
          textBaseline: 'top',
          fontFamily: attributes.fontFamily || 'PingFang SC'
        }, textStyle, {
          // 暂未支持，需判断是否可以和必要实现
          // textAlign,fontStyle,fontVariant,fontWeight,fontSize,lineHeight
          // Extra Attr
          showTip: showTip,
          fullText: text,
          // add link support
          link: link
        }),
        draggable: nodeDraggable
      }); // 如果文字被 clip，则展示相应 tooltip
      // https://g6.antv.vision/zh/examples/tool/tooltip#tooltipLocalCustom
    }

    return node;
  };
};

var updateText = function updateText(targetElement) {
  return function (node) {
    var _node$box2 = node.box,
        top = _node$box2.top,
        left = _node$box2.left;
    var _node$props2 = node.props,
        link = _node$props2.link,
        cursor = _node$props2.cursor; // 目前简化处理,仅处理第一个 text instance

    if (node && node.children && node.children[0] && node.children[0].value) {
      var text = node.children[0].value;
      var attributes = node.style || {};
      var textColor = attributes.color || '#000';
      var textStyle = {};

      if (attributes.fontSize) {
        Object.assign(textStyle, {
          fontSize: attributes.fontSize
        });
      }

      var commonStyle = {};

      if (attributes.cursor) {
        Object.assign(commonStyle, {
          cursor: attributes.cursor
        });
      }

      var showTip = false;
      var resultText = text;

      if (attributes.maxWidth || attributes.width) {
        resultText = clipText(text, attributes, attributes.maxWidth); // console.log(attributes, text, resultText, measureText(resultText, attributes));

        node.box.height = 48;
      }

      if (resultText !== text) {
        showTip = true;
      }

      targetElement.attr((0, _extends2["default"])({}, R.omit(['fontSize'], attributes), {
        // Common Attr
        fill: textColor,
        cursor: cursor || link && 'pointer'
      }, commonStyle, {
        // 暂未支持，需判断是否可以和必要实现
        // stroke,lineWidth,lineDash,shadowColor,shadowBlur,shadowOffsetX,shadowOffsetY,opacity,fillOpacity,strokeOpacity
        // Text Attr
        text: resultText,
        x: left,
        y: top,
        textBaseline: 'top',
        fontFamily: attributes.fontFamily || 'PingFang SC'
      }, textStyle, {
        // 暂未支持，需判断是否可以和必要实现
        // textAlign,fontStyle,fontVariant,fontWeight,fontSize,lineHeight
        // Extra Attr
        showTip: showTip,
        fullText: text
      })); // 如果文字被 clip，则展示相应 tooltip
      // https://g6.antv.vision/zh/examples/tool/tooltip#tooltipLocalCustom
    }

    return node;
  };
}; // export default R.curryN(2, renderText);


exports.updateText = updateText;
var _default = renderText;
exports["default"] = _default;