"use strict";

exports.__esModule = true;
exports.isAlphaNumber = exports.i18n = exports.getShadowStyle = exports.getOpacityStyle = exports.formatCssNumber = void 0;

var _lodash = require("lodash");

var localeTemp = (0, _lodash.get)(window, 'g_config.locale') || 'zh_CN';
var locale = /^zh[-_]cn$/i.test(localeTemp) ? 'zh_CN' : 'en_US';
var varList = (0, _lodash.get)(window, "arrangement[" + locale + "]") || {};

var i18n = function i18n(key) {
  return varList[key] || key;
};

exports.i18n = i18n;

var formatCssNumber = function formatCssNumber(value) {
  var val = +("" + value).replace('px', '');
  return isNaN(val) ? 0 : val;
};
/**
 * 判断是否合法的透明度值，范围是 [0, 1]
 * @param value Number 透明度值
 * @returns Boolean
 */


exports.formatCssNumber = formatCssNumber;

var isAlphaNumber = function isAlphaNumber(value) {
  return (0, _lodash.isNumber)(value) && value >= 0 && value <= 1;
};
/**
 * 获取元素的阴影样式
 * @param style IShadowStyle 阴影样式
 * @returns IShadowStyle
 */


exports.isAlphaNumber = isAlphaNumber;

var getShadowStyle = function getShadowStyle(style) {
  var shadowOffsetX = style.shadowOffsetX,
      shadowOffsetY = style.shadowOffsetY,
      shadowBlur = style.shadowBlur,
      shadowColor = style.shadowColor;
  var shadowStyle = {};

  if (shadowColor) {
    Object.assign(shadowStyle, {
      shadowColor: shadowColor,
      shadowBlur: formatCssNumber(shadowBlur || 0),
      shadowOffsetX: formatCssNumber(shadowOffsetX || 0),
      shadowOffsetY: formatCssNumber(shadowOffsetY || 0)
    });
  }

  return shadowStyle;
};
/**
 * 获取元素的透明度样式
 * @param style IOpacityStyle 透明度样式
 * @returns IOpacityStyle
 */


exports.getShadowStyle = getShadowStyle;

var getOpacityStyle = function getOpacityStyle(style) {
  var opacityStyle = {};
  Object.keys(style).forEach(function (key) {
    var value = style[key];

    if (isAlphaNumber(value)) {
      opacityStyle[key] = value;
    }
  });
  return opacityStyle;
};

exports.getOpacityStyle = getOpacityStyle;